@@ -20,6 +20,54 @@
 
 time_t timestamp;
 
+
+//added by gfhuang
+void MSN_sort_group(session)
+	MSN_session *session;
+{
+	int i, j;
+	MSN_group *temp;
+	int n;
+
+	n = session->num_groups;
+	for(i = 0; i < n; ++i) { 
+		for(j = i + 1; j < n; ++j) 
+			if(strcmp(session->group[j]->name, session->group[i]->name) < 0) {
+				temp = session->group[j];
+				session->group[j] = session->group[i];
+				session->group[i] = temp;
+			}
+#ifdef DEBUG
+		debug_log("group %d : %s %s\n", i, session->group[i]->name, session->group[i]->guid);
+#endif
+	}
+}
+int MSN_group_guid2index(session, guid) 
+	MSN_session *session;
+	char *guid;
+{		//added by gfhuang
+	int i;
+	for(i = 0; i < session->num_groups; ++i) {
+		if(0 != session->group[i]->guid[0] &&
+		  strstr(guid, session->group[i]->guid) != NULL) {
+			return i;
+		}
+	}
+	return -1;
+}
+int MSN_addr2index(session, addr)
+	MSN_session *session;
+	char *addr;
+{
+	int i;	
+        for (i = 0; i < session->num_contacts; i++)
+            if (strcmp(session->contact[i]->addr, addr) == 0)
+                break;
+	return i;
+}
+//added end
+
+
 void
 nonblock(sd, ret_sd)
      int sd;
@@ -177,11 +225,12 @@ MSN_conversation_sendmsg(sd, csc, str, session)
 }
 
 int
-MSN_conversation_handle(sd, num_ppl, message, message_len)
+MSN_conversation_handle(sd, num_ppl, message, message_len, session)
      int sd;
      int *num_ppl;
      char *message;
      int message_len;
+     MSN_session *session;		//added by gfhuang
 {
     char buf[1024], *ptr[4];
     int i, msg_len, header_len;
@@ -239,10 +288,9 @@ MSN_conversation_handle(sd, num_ppl, message, message_len)
                  */
                 if (strncmp(buf, "Content-Type", 12) == 0) {
                     if (strncmp(buf, "Content-Type: application/x-msnmsgrp2p", 38) == 0) {
-                        //MSN_handle_p2p_msg(sd);
+                        MSN_handle_p2p_msg(sd);
                         snprintf(message, message_len,
                                  "Received file-transfer invitation. Not supported yet.");
-                        free(ptr[0]);
                         return -1;
                     } else if (strncmp(buf, "Content-Type: text/plain", 24) != 0) {
                         free(ptr[0]);
@@ -284,8 +332,14 @@ MSN_conversation_handle(sd, num_ppl, message, message_len)
             convert_from_utf8(tr_buf, sizeof(tr_buf), ptr[1], msg_len);
 #endif
 
-            snprintf(message, message_len, "%s says:\n     %s", ptr[0],
-                     ptr[1]);
+	    //added by gfhuang
+	    i = MSN_addr2index(session, ptr[0]);
+	    if(i >= session->num_contacts) 
+	            snprintf(message, message_len, "%s says:\n     %s", ptr[0], ptr[1]);
+	    else
+	            snprintf(message, message_len, "%s says:\n     %s", session->contact[i]->name, ptr[1]);
+		
+
             free(ptr[0]);
             free(ptr[1]);
             return 2;
@@ -522,6 +576,15 @@ MSN_add_contact(addr, session, allow_or_block, errbuf, err_len)
                      session->csc++, addr);
             send(session->sd, buf, strlen(buf), 0);
         } else {
+            /*
+             * If on PL, then remove 
+             */
+            if ((PL & session->contact[i]->listnum) > 0) {
+                snprintf(buf, sizeof(buf) - 1, "REM %d PL %s\r\n",
+                         session->csc++, addr);
+                send(session->sd, buf, strlen(buf), 0);
+            }
+
             /*
              * If not on FL, then add 
              */
@@ -538,14 +601,6 @@ MSN_add_contact(addr, session, allow_or_block, errbuf, err_len)
                          "ADC %d RL N=%s\r\n", session->csc++, addr);
                 send(session->sd, buf, strlen(buf), 0);
             }
-            /*
-             * If on PL, then remove 
-             */
-            if ((PL & session->contact[i]->listnum) > 0) {
-                snprintf(buf, sizeof(buf) - 1, "REM %d PL %s\r\n",
-                         session->csc++, addr);
-                send(session->sd, buf, strlen(buf), 0);
-            }
 
             /*
              *  Add to AL or BL
@@ -695,6 +750,8 @@ MSN_server_handle(session, message, message_len)
         strncpy(tr_buf, buf, sizeof(tr_buf) - 1);
         convert_from_utf8(tr_buf, sizeof(tr_buf), buf, sizeof(buf));
 #endif
+
+
         if ((ptr[0] = strstr(buf, "\r\n")) != NULL)
             *ptr[0] = '\0';
 
@@ -757,10 +814,19 @@ MSN_server_handle(session, message, message_len)
                 j = 13;
 
             session->contact[i]->status = MSN_status2int(ptr[0]);
-            snprintf(message, message_len - 1,
+	    if(0 == session->contact[i]->psm[0]) 
+                snprintf(message, message_len - 1,
                      "%s (%s) is %s",
                      session->contact[i]->name,
-                     ptr[1], MSN_status2str(session->contact[i]->status));
+                     ptr[1], 
+		     MSN_status2str(session->contact[i]->status));
+	    else //added by gfhuang
+                snprintf(message, message_len - 1,
+                     "%s (%s) {%s} is %s",
+                     session->contact[i]->name,
+                     ptr[1], 
+		     session->contact[i]->psm,
+		     MSN_status2str(session->contact[i]->status));
             for (i = 0; i < 3; i++)
                 free(ptr[i]);
             return j;           /* 13 or 14 */
@@ -771,17 +837,57 @@ MSN_server_handle(session, message, message_len)
             /*
              * but do not do anything with it      
              */
-            if ((ptr[0] = (char *)split(buf, ' ', 2)) == NULL) {
+            if ((ptr[1] = (char *)split(buf, ' ', 1)) == NULL ||	//by gfhuang
+		(ptr[0] = (char *)split(buf, ' ', 2)) == NULL) {
                 strncpy(message, "Couldn't parse UBX", message_len - 1);
                 return -1;
             }
             i = atoi(ptr[0]);
             free(ptr[0]);
-            if (read(session->sd, buf, i) != i) {
+
+	    if (read(session->sd, buf, i) != i) {
                 strncpy(message, "Couldn't read UBX payload",
                         message_len - 1);
                 return -1;
             }
+	    // parsing PSM, by gfhuang
+	    if(0 == i) buf[0] = 0;	//important, by gfhuang, when i=0, buf is untouched!
+
+	    i = MSN_addr2index(session, ptr[1]);
+	    free(ptr[1]);
+	    if(i >= session->num_contacts) {
+                strncpy(message, "Couldn't find UBX contact in your list",
+                        message_len - 1);
+		return -1;
+	    }
+	    session->contact[i]->psm[0] = 0;
+	    if(0 == buf[0]) return 0; 
+
+	    if((ptr[0] = strstr(buf, "<PSM>")) != NULL && NULL != (ptr[1] = strstr(ptr[0], "</PSM>"))) {
+		ptr[0] += 5;
+		*ptr[1] = 0;
+	    }
+	    else {
+                strncpy(message, "error in parsing PSM",
+                        message_len - 1);
+#ifdef DEBUG
+		debug_log("unparsed psm: %s\n", buf);
+#endif
+		return -1;
+	    }
+	    if(0 != *ptr[0]) {	//convert_from_utf8 will fail when len==0 ????
+#ifdef HAVE_ICONV
+	    	    convert_from_utf8(ptr[0], strlen(ptr[0]), session->contact[i]->psm, PSM_LEN - 1);
+#else
+		    strncpy(session->contact[i]->psm, ptr[0], PSM_LEN - 1);
+#endif
+            	    snprintf(message, message_len-1, "%s's psm : %s", session->contact[i]->addr,  
+    	    						session->contact[i]->psm);
+		    return -1;   // has message
+	    }
+//	    else {
+//		    strncpy(session->contact[i]->psm, "<empty>", PSM_LEN - 1);
+//	    }
             return 0;
         } else if (strncmp(buf, "QNG", 3) == 0)
             return 0;
@@ -1033,6 +1139,8 @@ wait_for_input(int fd, unsigned int seconds)
     return select(FD_SETSIZE, &set, NULL, NULL, &timeout);
 }
 
+
+
 int
 MSN_load_userlist(session, cb)
      MSN_session *session;
@@ -1040,12 +1148,12 @@ MSN_load_userlist(session, cb)
 {
     char buf[512], tr_buf[256];
     char *ptr;
-    int i, a;
+    int i, a, curGroup;
 
     /*
      * clear the buffer 
      */
-    while (wait_for_input(session->sd, 1) != 0)
+    while (wait_for_input(session->sd, 1) > 0)
         recv(session->sd, buf, sizeof(buf) - 1, 0);
 
     snprintf(buf, sizeof(buf) - 1, "SYN %d 0 0\r\n", session->csc++);
@@ -1062,6 +1170,18 @@ MSN_load_userlist(session, cb)
     session->num_contacts = atoi(ptr);
     free(ptr);
 
+    // groups, added by gfhuang
+    if ((ptr = split(buf, ' ', 5)) == NULL)
+        return -1;
+
+    session->num_groups = atoi(ptr);
+    free(ptr);
+    MSN_resize_group_array(session);
+    for(i = 0; i <= session->num_groups; ++i)
+	session->group[i] = MSN_allocate_group();
+    curGroup = 0;
+    //added end
+
     /*
      * allocate space for the contacts 
      */
@@ -1080,6 +1200,10 @@ MSN_load_userlist(session, cb)
         if ((ptr = strstr(buf, "\r\n")) != NULL)
             *ptr = 0x0;
 
+#ifdef DEBUG
+	debug_log("ld : %s\n", buf);
+#endif
+
         if (strncmp(buf, "GTC", 3) == 0) {
             /*
              * Store the char 'A' or 'B' in gtc 
@@ -1097,7 +1221,23 @@ MSN_load_userlist(session, cb)
                 convert_from_utf8(tr_buf, sizeof(tr_buf) - 1,
                                   session->me.name, NAME_LEN - 1);
             #endif
-        } else if (strncmp(buf, "LST", 3) == 0) {
+        } else if (strncmp(buf, "LSG", 3) == 0) {		//group, added by gfhuang
+		ptr = split(buf, ' ', 1);
+		if(NULL != ptr) {
+			strncpy(session->group[curGroup]->name, ptr, NAME_LEN - 1);
+			free(ptr);
+
+			ptr = split(buf, ' ', 2);
+			if(NULL != ptr) {
+				strncpy(session->group[curGroup]->guid, ptr, GUID_LEN - 1);
+				++curGroup;
+				free(ptr);
+			}
+		}
+
+	} else if (strncmp(buf, "LST", 3) == 0) {
+	    if(0 == i) MSN_sort_group(session);	//added by gfhuang, first time, sort the group
+
             for (a = 1; (ptr = split(buf, ' ', a)) != NULL; a++) {
                 if (strncmp(ptr, "N=", 2) == 0)
                     strncpy(session->contact[i]->addr, &ptr[2], ADDR_LEN - 1);
@@ -1111,8 +1251,10 @@ MSN_load_userlist(session, cb)
                     #endif
                 } else if (strncmp(ptr, "C=", 2) == 0)
                     strncpy(session->contact[i]->guid, &ptr[2], GUID_LEN - 1);
-                else if (isdigit(ptr[0]))
+                else if (isdigit(ptr[0]) && NULL == strstr(ptr, "-")) //thanks poppyer
                     session->contact[i]->listnum = atoi(ptr);
+		else		//group_guids, added by gfhuang 
+		    session->contact[i]->ingroup = MSN_group_guid2index(session, ptr);
 
                 free(ptr);
             }
