@@ -24,12 +24,12 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-int debug_fd;
+FILE *debug_fd;
 
-int
+FILE *
 debug_init(void)
 {
-    debug_fd = open("DEBUG.txt", O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
+    debug_fd = fopen("DEBUG.txt", "w");
     return debug_fd;
 }
 
@@ -42,7 +42,9 @@ debug_log(char *fmt, ...)
     va_start(ap, fmt);
     vasprintf(&ptr, fmt, ap);
     va_end(ap);
-    write(debug_fd, ptr, strlen(ptr));
+    fprintf(debug_fd, "%s\n", ptr);
+    //write(debug_fd, ptr, strlen(ptr));
+    fflush(debug_fd);
     free(ptr);
     return 0;
 }
@@ -50,5 +52,5 @@ debug_log(char *fmt, ...)
 void
 debug_destroy(void)
 {
-    close(debug_fd);
+    fclose(debug_fd);
 }
