@@ -18,11 +18,10 @@
 #include "common.h"
 
 int
-UI_handle_mouse(ui, menu, session, cf, focus)
+UI_handle_mouse(ui, menu, session, focus)
      struct ui *ui;
      struct menu *menu;
      MSN_session *session;
-     config *cf;
      int *focus;
 {
     MEVENT mouse;
@@ -42,11 +41,11 @@ UI_handle_mouse(ui, menu, session, cf, focus)
                   j=UI_menu_cntlines(menu->menu[menu->c_col]);
                   if(my<=j) {
                       menu->c_row=my-1;
-                      if ((e = UI_menu_handle_key(menu, cf, '\n')) == cmd_quit) {
+                      if ((e = UI_menu_handle_key(menu, '\n')) == cmd_quit) {
                           if (UI_dialog_getch("Are you sure you want to quit? (y/n)") == 'y')
                               return -1;
                       } else {
-                          r = UI_do_command(session, cf, ui, e, buf, sizeof(buf) - 1);
+                          r = UI_do_command(session, ui, e, buf, sizeof(buf) - 1);
                           if (r < 0) {
                               wattron(ui->main.swin, COLOR_PAIR(1) | A_BOLD);
                               UI_win_addstr(ui, &ui->main, "[%s] ",
@@ -55,45 +54,45 @@ UI_handle_mouse(ui, menu, session, cf, focus)
                               UI_win_addstr(ui, &ui->main, "Error: %s\n", buf);
                           }
                       }
-                      UI_remove_focus(ui, menu, cf, *focus);
+                      UI_remove_focus(ui, menu, *focus);
                       *focus = FOCUS_MAIN;
-                      UI_set_focus(ui, menu, cf, *focus);
+                      UI_set_focus(ui, menu, *focus);
                       return 1;
                 }
             }
             //Main click
             if ((mx < (wx-LIST_WIDTH))&&
                 (my > 0)&&(my <= (wy-7))) {
-                UI_remove_focus(ui, menu, cf, *focus);
+                UI_remove_focus(ui, menu, *focus);
                 *focus = FOCUS_MAIN;
-                UI_set_focus(ui, menu, cf, *focus);
+                UI_set_focus(ui, menu, *focus);
                 return 1;
             }
             //Menu click
             if (my == 0) {
                 int i;
-                UI_remove_focus(ui, menu, cf, *focus);
+                UI_remove_focus(ui, menu, *focus);
                 *focus = FOCUS_MENU;
-                UI_set_focus(ui, menu, cf, *focus);
+                UI_set_focus(ui, menu, *focus);
                 for(i=0;i<((mx-mx%menu->titlew)/menu->titlew);i++) {
                     UI_menu_process_cmd(menu, cmd_right);
                 }
                 UI_menu_process_cmd(menu, cmd_choose);
-                UI_menu_draw(menu, cf);
+                UI_menu_draw(menu);
                 return 1;
             }
             //Contact list click
             if ((mx >= wx-LIST_WIDTH)&&(my < 2*(wy-3)/3)) {
-                UI_remove_focus(ui, menu, cf, *focus);
+                UI_remove_focus(ui, menu, *focus);
                 *focus = FOCUS_LIST;
-                UI_set_focus(ui, menu, cf, *focus);
+                UI_set_focus(ui, menu, *focus);
                 return 1;
             }
             //Conversation list click
             if((mx >= wx-LIST_WIDTH)&&(my >= 2*(wy-3)/3)) {
-                UI_remove_focus(ui, menu, cf, *focus);
+                UI_remove_focus(ui, menu, *focus);
                 *focus = FOCUS_CONV;
-                UI_set_focus(ui, menu, cf, *focus);
+                UI_set_focus(ui, menu, *focus);
                 return 1;
             }
         }
@@ -102,19 +101,18 @@ UI_handle_mouse(ui, menu, session, cf, focus)
 }
 
 int
-UI_handle_server_code(ui, session, c, str, cf)
+UI_handle_server_code(ui, session, c, str)
      struct ui *ui;
      MSN_session *session;
      int c;
      char *str;
-     config *cf;
 {
     char *ptr[2] = { NULL, NULL }, t_buf[16];
     int r;
 
     switch (c) {
     case -2:                   /* fatal error */
-        UI_play_sound(cf, SOUND_ERROR);
+        UI_play_sound(SOUND_ERROR);
         UI_err_exit("Quitting TMSNC: %s\n", str);
         break;
     case 2:                    /* initial login */
@@ -122,13 +120,13 @@ UI_handle_server_code(ui, session, c, str, cf)
     case 11:                   /* added contact */
     case 12:                   /* removed contact */
     case 13:                   /* someone changed status */
-        UI_draw_list(ui, session, cf);
+        UI_draw_list(ui, session);
         break;
     case 8:                    /* we are invited to a conversation */
         if ((ptr[0] = split(str, ' ', 0)) == NULL ||
             (ptr[1] = split(str, ' ', 1)) == NULL)
             UI_err_exit("Cannot split conversation string");
-        r = UI_start_conversation(ui, session, cf, ptr[1], atoi(ptr[0]));
+        r = UI_start_conversation(ui, session, ptr[1], atoi(ptr[0]));
         free(ptr[0]);
         free(ptr[1]);
 
@@ -138,17 +136,17 @@ UI_handle_server_code(ui, session, c, str, cf)
                           UI_get_timestr(t_buf, sizeof(t_buf) - 1));
             wattroff(ui->main.swin, COLOR_PAIR(1) | A_BOLD);
             UI_win_addstr(ui, &ui->main,
-                          "[%s] Error: Cannot accept conversation\n");
-            UI_play_sound(cf, SOUND_ERROR);
+                          "Error: Cannot accept conversation\n");
+            UI_play_sound(SOUND_ERROR);
         }
-        UI_draw_conv(ui, cf);
+        UI_draw_conv(ui);
         break;
     case 9:                    /* we changed screen-name */
         UI_bar_draw(session);
         break;
     case 14:                   /* someone logged in */
-        UI_play_sound(cf, SOUND_LOGIN);
-        UI_draw_list(ui, session, cf);
+        UI_play_sound(SOUND_LOGIN);
+        UI_draw_list(ui, session);
         break;
     default:
         break;
@@ -157,10 +155,9 @@ UI_handle_server_code(ui, session, c, str, cf)
 }
 
 int
-UI_handle_conversation_code(ui, session, cf, conv, c, message)
+UI_handle_conversation_code(ui, session, conv, c, message)
      struct ui *ui;
      MSN_session *session;
-     config *cf;
      struct conv *conv;
      int c;
      char *message;
@@ -177,7 +174,7 @@ UI_handle_conversation_code(ui, session, cf, conv, c, message)
                                      sizeof(char *));
         conv->ppl[conv->num_ppl-1] = tmp;
         strncpy(conv->last_contact, tmp, CONTACT_LEN - 1);
-        UI_draw_conv(ui, cf);
+        UI_draw_conv(ui);
         do_print_msg++;
         break;
     case 3:                    /* BYE (due to inactivity) */
@@ -193,11 +190,11 @@ UI_handle_conversation_code(ui, session, cf, conv, c, message)
               conv->ppl[i] = conv->ppl[i+1];
         }
         free(tmp);
-        UI_draw_conv(ui, cf);
+        UI_draw_conv(ui);
         do_print_msg++;
         break;
     case -2:                   /* Connection lost */
-        UI_close_conversation(ui, cf, conv);
+        UI_close_conversation(ui, conv);
 
         wattron(ui->main.swin, COLOR_PAIR(1) | A_BOLD);
         UI_win_addstr(ui, &(ui->main), "[%s] ",
@@ -217,8 +214,8 @@ UI_handle_conversation_code(ui, session, cf, conv, c, message)
     if (do_print_msg) {
         if (ui->c_conv != conv) {
             conv->new_message = 1;
-            UI_draw_conv(ui, cf);
-            UI_play_sound(cf, SOUND_NEWMSG);
+            UI_draw_conv(ui);
+            UI_play_sound(SOUND_NEWMSG);
         }
         wattron(conv->win.swin, (c < 0) ? COLOR_PAIR(1) : COLOR_PAIR(2));
         UI_win_addstr(ui, &(conv->win), "[%s]",
@@ -233,10 +230,9 @@ UI_handle_conversation_code(ui, session, cf, conv, c, message)
 }
 
 int
-UI_main_handle_key(ui, session, cf, c)
+UI_main_handle_key(ui, session, c)
      struct ui *ui;
      MSN_session *session;
-     config *cf;
      int c;
 {
     char t_buf[16];
@@ -301,20 +297,25 @@ UI_main_handle_key(ui, session, cf, c)
             if (i == session->num_contacts)
                 return -1;
             if (session->contact[i]->status != 7) {
-                MSN_conversation_call(ui->c_conv->sd, &(ui->c_conv->csc),
-                                      ui->c_conv->last_contact);
+		// sent msg crash after long left conversation bug, by gfhuang
+		ui->conv->sd = MSN_conversation_initiate(session, ui->c_conv->last_contact, NULL);
+		if(ui->conv->sd < 0) return -1;
+		ui->conv->csc = 2;
+	
+//                MSN_conversation_call(ui->c_conv->sd, &(ui->c_conv->csc),
+//                                      ui->c_conv->last_contact);
 
                 /*
                  * to give the server some time to process
                  * our invitation
                  */
-                sleep(1);
+//                sleep(1);
             }
         }
         /*
          * Run the buffer through a talkfilter if available 
          */
-        UI_filter_translate(cf, c_win->field_buf, FIELDBUFSIZE - 1);
+        UI_filter_translate(c_win->field_buf, FIELDBUFSIZE - 1);
 
         /*
          * Send the message 
@@ -369,9 +370,8 @@ UI_main_handle_key(ui, session, cf, c)
 }
 
 int
-UI_do_command(session, cf, ui, c, ret, retsize)
+UI_do_command(session, ui, c, ret, retsize)
      MSN_session *session;
-     config *cf;
      struct ui *ui;
      int c;
      char *ret;
@@ -446,7 +446,7 @@ UI_do_command(session, cf, ui, c, ret, retsize)
         if (UI_dialog_getstr
             ("Name of filter ('none' to disable)", buf,
              sizeof(buf) - 1) == 0)
-            if (UI_set_filter(cf, buf) < 0)
+            if (UI_set_filter(buf) < 0)
                 UI_dialog_getch("No such filter, read the manual");
 #else
         UI_dialog_getch("TMSNC was not compiled with talkfilter support");
@@ -493,18 +493,18 @@ UI_do_command(session, cf, ui, c, ret, retsize)
         break;
     case cmd_setcustomnick:
         ptr = UI_get_contact_name(session->contact[UI_get_list_pointer()],
-                                  cf->use_nickname);
+                                  UI_get_config()->use_nickname);
         snprintf(r_buf, sizeof(r_buf) - 1, "New custom nickname for %s",
                  ptr);
         free(ptr);
 
         if (UI_dialog_getstr(r_buf, buf, sizeof(buf) - 1) == 0)
             UI_set_alias(session->contact[UI_get_list_pointer()], buf);
-        UI_draw_list(ui, session, cf);
+        UI_draw_list(ui, session);
         break;
     case cmd_unsetcustomnick:
         UI_delete_alias(session->contact[UI_get_list_pointer()]);
-        UI_draw_list(ui, session, cf);
+        UI_draw_list(ui, session);
         break;
     case cmd_manual:
         def_prog_mode();
@@ -529,10 +529,9 @@ UI_do_command(session, cf, ui, c, ret, retsize)
 
 //Start of iveqy
 int
-UI_handle_power_functions(ui, session, cf)
+UI_handle_power_functions(ui, session)
      struct ui *ui;
      MSN_session *session;
-     config *cf;
 {
     int i;
     char r_buf[512];
@@ -554,7 +553,7 @@ UI_handle_power_functions(ui, session, cf)
     } else if (!strcasecmp("remove all", r_buf)) {
         if (UI_dialog_getch("Remove entire contact list? (y/n)") == 'y') {
             for (i = 0; i < session->num_contacts; i++)
-                MSN_block_contact(session->contact[i]->addr, session, r_buf,
+                MSN_remove_contact(session->contact[i], session, r_buf,
                                   sizeof(r_buf) - 1);
         }
     } else if (!strcasecmp("whoami", r_buf)) {
@@ -597,41 +596,40 @@ UI_handle_power_functions(ui, session, cf)
 }
 
 int
-UI_handle_altkey(ui, menu, cf, focus)
+UI_handle_altkey(ui, menu, focus)
      struct ui *ui;
      struct menu *menu;
-     config *cf;
      int *focus;
 {
     int c;
 
     switch ((c = tolower(getch()))) {
     case 'a':
-        UI_remove_focus(ui, menu, cf, *focus);
+        UI_remove_focus(ui, menu, *focus);
         *focus = 1;
-        UI_set_focus(ui, menu, cf, *focus);
+        UI_set_focus(ui, menu, *focus);
         break;
     case 's':
-        UI_remove_focus(ui, menu, cf, *focus);
+        UI_remove_focus(ui, menu, *focus);
         *focus = 2;
-        UI_set_focus(ui, menu, cf, *focus);
+        UI_set_focus(ui, menu, *focus);
         break;
     case 'x':
-        UI_remove_focus(ui, menu, cf, *focus);
+        UI_remove_focus(ui, menu, *focus);
         *focus = 3;
-        UI_set_focus(ui, menu, cf, *focus);
+        UI_set_focus(ui, menu, *focus);
         break;
     case 'z':
-        UI_remove_focus(ui, menu, cf, *focus);
+        UI_remove_focus(ui, menu, *focus);
         *focus = 4;
-        UI_set_focus(ui, menu, cf, *focus);
+        UI_set_focus(ui, menu, *focus);
         break;
     case 'c':
         /*
          * close current conversation 
          */
         if (ui->c_conv != NULL) {
-            UI_close_conversation(ui, cf, ui->c_conv);
+            UI_close_conversation(ui, ui->c_conv);
         }
         break;
     case 'w':
@@ -652,7 +650,7 @@ UI_handle_altkey(ui, menu, cf, focus)
                 UI_current_conversation(ui, &(ui->conv[c]));
                 ui->conv[c].new_message = 0;
 
-                UI_draw_conv(ui, cf);
+                UI_draw_conv(ui);
             }
         }
         break;
