@@ -17,5 +17,14 @@
 #include <string.h>
 #include <stdlib.h>
 
+enum list_show_mode {
+	list_show_default,
+	list_show_addr,
+	list_show_nick,
+	list_show_psm,
+	list_show_END
+};
+
 int list_offset = 0, list_pointer = 0;
 int conv_offset = 0, conv_pointer = 0;
+int list_show = list_show_default; 
